#include "tinypb_channel.hpp"
#include "error_code.hpp"
#include "logger.hpp"
#include "tinypb_controller.hpp"
#include "tinypb_data.hpp"
#include <google/protobuf/message.h>
#include <memory>



namespace tinyrpc {
    TinypbChannel::TinypbChannel(const NetAddress& peerAddr) : 
        m_client(peerAddr)
    {


    }

    void TinypbChannel::CallMethod(const google::protobuf::MethodDescriptor* method,
        google::protobuf::RpcController* controller,
        const google::protobuf::Message* request,
        google::protobuf::Message* response,
        google::protobuf::Closure* done)  
    {

        TinypbController* rpc_controller = dynamic_cast<TinypbController*>(controller);
        rpc_controller->SetLocalAddr(m_client.getLocalAddr());
        rpc_controller->SetPeerAddr(m_client.getPeerAddr());

        std::unique_ptr<TinypbData> data(new TinypbData);

        data->service_full_name = method->full_name();

        if (!request->SerializeToString(&(data->pb_data))) {
            logger() << "serialize send package error";
            return;
        }
 
        data->msg_req = TinypbData::genMsgNumber(); 
        data->msg_req_len = data->msg_req.length();
        rpc_controller->SetMsgReq(data->msg_req);

        m_client.writeToSendBuffer(*data);

        std::shared_ptr<AbstractData> res;

        int ret = m_client.sendAndRecvData(data->msg_req, res);

        if(ret != 0) {
            rpc_controller->SetError(ret, "sendAndRecvData err");
            return;
        }
     
        std::shared_ptr<TinypbData> res_data = std::dynamic_pointer_cast<TinypbData>(res);
        // CONTINUE
        if(!response->ParseFromString(res_data->pb_data)) {
            rpc_controller->SetError(ERROR_FAILED_DESERIALIZE, "failed to deserialize data from server");  
            return;
        }

        if(res_data->err_code != 0) {
            rpc_controller->SetError(res_data->err_code, res_data->err_info);
            return;
        }

        logger() << "============================================================";
        logger() << data->msg_req << "|" << rpc_controller->PeerAddr().toString()
            << "|call rpc server [" << data->service_full_name << "] succ" 
            << ". Get server reply response data:" << response->ShortDebugString();
        logger() << "============================================================";

        if(done) done->Run();
 
    }


}